﻿using FlowerShopBusinessLogic.OfficePackage;
using FlowerShopBusinessLogic.OfficePackage.HelperModels;
using FlowerShopContracts.BindingModels;
using FlowerShopContracts.BusinessLogicsContracts;
using FlowerShopContracts.SearchModels;
using FlowerShopContracts.StoragesContracts;
using FlowerShopContracts.ViewModels;

namespace FlowerShopBusinessLogic.BusinessLogics
{
    public class ReportLogic : IReportLogic
    {
        private readonly IComponentStorage _componentStorage;
        private readonly IBouquetStorage _bouquetStorage;
        private readonly IOrderStorage _orderStorage;
        private readonly IShopStorage _shopStorage;
        private readonly AbstractSaveToExcel _saveToExcel;
        private readonly AbstractSaveToWord _saveToWord;
        private readonly AbstractSaveToPdf _saveToPdf;

        public ReportLogic(IBouquetStorage bouquetStorage, IComponentStorage componentStorage, IShopStorage shopStorage, IOrderStorage orderStorage, AbstractSaveToExcel saveToExcel, AbstractSaveToWord saveToWord, AbstractSaveToPdf saveToPdf)
        {
            _bouquetStorage = bouquetStorage;
            _componentStorage = componentStorage;
            _orderStorage = orderStorage;
            _shopStorage = shopStorage;
            _saveToExcel = saveToExcel;
            _saveToWord = saveToWord;
            _saveToPdf = saveToPdf;
        }
        
        public List<ReportBouquetComponentViewModel> GetBouquetComponents()
        {
            var bouquets = _bouquetStorage.GetFullList();
            var list = new List<ReportBouquetComponentViewModel>();
            foreach (var bouquet in bouquets)
            {
                var record = new ReportBouquetComponentViewModel
                {
                    BouquetName = bouquet.BouquetName,
                    Components = new List<(string Component, int Count)>(),
                    TotalCount = 0
                };
                foreach (var component in bouquet.BouquetComponents.Values)
                {
                    record.Components.Add((component.Item1.ComponentName, component.Item2));
                    record.TotalCount += component.Item2;
                }
                list.Add(record);
            }
            return list;
        }

        public List<ReportOrdersViewModel> GetOrders(ReportBindingModel model)
        {
            return _orderStorage.GetFilteredList(new OrderSearchModel
            {
                DateFrom = model.DateFrom,
                DateTo = model.DateTo
            })
            .Select(x => new ReportOrdersViewModel
            {
                Id = x.Id,
                DateCreate = x.DateCreate,
                BouquetName = x.BouquetName,
                Sum = x.Sum,
                Status = x.Status.ToString(),
            })
           .ToList();
        }

        public void SaveBouquetsToWordFile(ReportBindingModel model)
        {
            _saveToWord.CreateDoc(new WordInfo
            {
                FileName = model.FileName,
                Title = "Список букетов",
                Bouquets = _bouquetStorage.GetFullList()
            });
        }
        
        public void SaveBouquetComponentToExcelFile(ReportBindingModel model)
        {
            _saveToExcel.CreateReport(new ExcelInfo
            {
                FileName = model.FileName,
                Title = "Список букетов",
                BouquetComponents = GetBouquetComponents()
            });
        }
        
        public void SaveOrdersToPdfFile(ReportBindingModel model)
        {
            _saveToPdf.CreateDoc(new PdfInfo
            {
                FileName = model.FileName,
                Title = "Список заказов",
                DateFrom = model.DateFrom!.Value,
                DateTo = model.DateTo!.Value,
                Orders = GetOrders(model)
            });
        }

        public void SaveShopsToWordFile(ReportBindingModel model)
        {
            _saveToWord.CreateTableDoc(new WordInfo
            {
                FileName = model.FileName,
                Title = "Список магазинов",
                Shops = _shopStorage.GetFullList()
            });
        }

        public void SaveShopBouquetsToExcelFile(ReportBindingModel model)
        {
            _saveToExcel.CreateShopReport(new ExcelInfo
            {
                FileName = model.FileName,
                Title = "Загруженность магазинов",
                ShopBouquets = GetShopBouquets()
            });
        }

        public List<ReportShopBouquetsViewModel> GetShopBouquets()
        {
            var shops = _shopStorage.GetFullList();
            var list = new List<ReportShopBouquetsViewModel>();

            foreach (var shop in shops)
            {
                var record = new ReportShopBouquetsViewModel
                {
                    ShopName = shop.Name,
                    Bouquets = new List<(string, int)>(),
                    Count = 0
                };

                foreach (var bouquetCount in shop.ShopBouquets.Values)
                {
                    record.Bouquets.Add((bouquetCount.Item1.BouquetName, bouquetCount.Item2));
                    record.Count += bouquetCount.Item2;
                }

                list.Add(record);
            }

            return list;
        }

        public List<ReportDateOrdersViewModel> GetDateOrders()
        {
            return _orderStorage.GetFullList().GroupBy(x => x.DateCreate.Date).Select(x => new ReportDateOrdersViewModel
            {
                DateCreate = x.Key,
                CountOrders = x.Count(),
                SumOrders = x.Sum(y => y.Sum)
            }).ToList();
        }

        public void SaveDateOrdersToPdfFile(ReportBindingModel model)
        {
            _saveToPdf.CreateReportDateDoc(new PdfInfo
            {
                FileName = model.FileName,
                Title = "Заказы по датам",
                DateOrders = GetDateOrders()
            });
        }
    }
}
